@@ -31,17 +31,11 @@
 #ifndef __NAUTILUS_ACTIONS_API_NA_IFACTORY_PROVIDER_PROVIDER_H__
 #define __NAUTILUS_ACTIONS_API_NA_IFACTORY_PROVIDER_PROVIDER_H__
 
-/**
- * SECTION: na_ifactory_provider
- * @short_description: #NAIFactoryProvider interface definition.
- * @include: nautilus-actions/na-ifactory_provider-fn.h
- */
-
 #include <glib.h>
 
 G_BEGIN_DECLS
 
-typedef struct NAIFactoryProvider                 NAIFactoryProvider;
+typedef struct _NAIFactoryProvider NAIFactoryProvider;
 
 G_END_DECLS
 
