@@ -50,7 +50,7 @@
 
 /* private interface data
  */
-struct NAIContextInterfacePrivate {
+struct _NAIContextInterfacePrivate {
 	void *empty;						/* so that gcc -pedantic is happy */
 };
 
@@ -61,6 +61,8 @@ static GType    register_type( void );
 static void     interface_base_init( NAIContextInterface *klass );
 static void     interface_base_finalize( NAIContextInterface *klass );
 
+static void     convert_allfiles_mimetype( NAIContext *context );
+
 static gboolean v_is_candidate( NAIContext *object, guint target, GList *selection );
 
 static gboolean is_candidate_for_target( const NAIContext *object, guint target, GList *files );
@@ -170,19 +172,21 @@ interface_base_finalize( NAIContextInterface *klass )
  * na_icontext_is_candidate:
  * @context: a #NAIContext to be checked.
  * @target: the current target.
- * @selection: the currently selected items, as a #GList of #NASelectedInfo items.
+ * @selection: the currently selected items, as a #GList of NASelectedInfo items.
  *
  * Determines if the given object may be candidate to be displayed in
  * the Nautilus context menu, depending of the list of currently selected
  * items.
  *
- * This function is called by nautilus-actions::build_nautilus_menus()
- * for each item found in #NAPivot items list, and, when this an action,
- * for each profile of this action.
+ * This function is called by <methodname>build_nautilus_menus</methodname>
+ * plugin function for each item found in NAPivot items list, and, when this
+ * an action, for each profile of this action.
  *
  * Returns: %TRUE if this @context succeeds to all tests and is so a
  * valid candidate to be displayed in Nautilus context menu, %FALSE
  * else.
+ *
+ * Since: 2.30
  */
 gboolean
 na_icontext_is_candidate( const NAIContext *context, guint target, GList *selection )
@@ -222,11 +226,13 @@ na_icontext_is_candidate( const NAIContext *context, guint target, GList *select
  *
  * Returns: %TRUE if this @context is valid, %FALSE else.
  *
- * This function is part of #NAIDuplicable::check_status() and is called
- * by #NAIDuplicable objects which also implement #NAIContext
+ * This function is part of <methodname>NAIDuplicable::check_status</methodname>
+ * and is called by #NAIDuplicable objects which also implement #NAIContext
  * interface. It so doesn't make sense of asking the object for its
  * validity status as it has already been checked before calling the
  * function.
+ *
+ * Since: 2.30
  */
 gboolean
 na_icontext_is_valid( const NAIContext *context )
@@ -250,6 +256,8 @@ na_icontext_is_valid( const NAIContext *context )
  * @context: the #NAIContext object to be checked.
  *
  * Returns: %TRUE if this @context is valid for all mimetypes, %FALSE else.
+ *
+ * Since: 2.30
  */
 gboolean
 na_icontext_is_all_mimetypes( const NAIContext *context )
@@ -278,6 +286,8 @@ na_icontext_is_all_mimetypes( const NAIContext *context )
 		break;
 	}
 
+	na_core_utils_slist_free( mimetypes );
+
 	return( is_all );
 }
 
@@ -286,10 +296,28 @@ na_icontext_is_all_mimetypes( const NAIContext *context )
  * @context: the #NAIContext to be prepared.
  *
  * Prepares the specified #NAIContext just after it has been readen.
+ *
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>
+ *       This converts a 'all/allfiles' mimetype to 'all/all' + 'file' scheme.
+ *     </para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>
+ *       This setup an internal flag when mimetypes is like 'all/all'
+ *       in order to optimize computation time;
+ *     </para>
+ *   </listitem>
+ * </itemizedlist>
+ *
+ * Since: 2.30
  */
 void
 na_icontext_read_done( NAIContext *context )
 {
+	convert_allfiles_mimetype( context );
+
 	na_object_set_all_mimetypes( context, na_icontext_is_all_mimetypes( context ));
 }
 
@@ -300,6 +328,8 @@ na_icontext_read_done( NAIContext *context )
  * @selected: whether this scheme is candidate to this @context.
  *
  * Sets the status of a @scheme relative to this @context.
+ *
+ * Since: 2.30
  */
 void
 na_icontext_set_scheme( NAIContext *context, const gchar *scheme, gboolean selected )
@@ -321,6 +351,8 @@ na_icontext_set_scheme( NAIContext *context, const gchar *scheme, gboolean selec
  * @selected: whether this @desktop is candidate to this @context.
  *
  * Sets the status of the @desktop relative to this @context for the OnlyShowIn list.
+ *
+ * Since: 2.30
  */
 void
 na_icontext_set_only_desktop( NAIContext *context, const gchar *desktop, gboolean selected )
@@ -342,6 +374,8 @@ na_icontext_set_only_desktop( NAIContext *context, const gchar *desktop, gboolea
  * @selected: whether this @desktop is candidate to this @context.
  *
  * Sets the status of the @desktop relative to this @context for the NotShowIn list.
+ *
+ * Since: 2.30
  */
 void
 na_icontext_set_not_desktop( NAIContext *context, const gchar *desktop, gboolean selected )
@@ -363,6 +397,8 @@ na_icontext_set_not_desktop( NAIContext *context, const gchar *desktop, gboolean
  * @new: the new uri.
  *
  * Replaces the @old URI by the @new one.
+ *
+ * Since: 2.30
  */
 void
 na_icontext_replace_folder( NAIContext *context, const gchar *old, const gchar *new )
@@ -378,6 +414,55 @@ na_icontext_replace_folder( NAIContext *context, const gchar *old, const gchar *
 	na_core_utils_slist_free( folders );
 }
 
+/*
+ * Convert 'all/allfiles' mimetype to 'all/all' + 'file' scheme.
+ * This takes into account
+ * - all/allfiles, allfiles, allfiles/ * and allfiles/all
+ * - negated assertions.
+ */
+static void
+convert_allfiles_mimetype( NAIContext *context )
+{
+	GSList *mimetypes, *im;
+	GSList *schemes;
+	gchar *tmp;
+	gboolean modified;
+
+	modified = FALSE;
+	mimetypes = na_object_get_mimetypes( context );
+	schemes = na_object_get_schemes( context );
+
+	for( im = mimetypes ; im ; im = im->next ){
+		if( !im->data || !strlen( im->data )){
+			continue;
+		}
+
+		gchar *imtype = ( gchar * ) im->data;
+		gboolean positive = is_positive_assertion( imtype );
+		guint i = ( positive ? 0 : 1 );
+
+		if( !strcmp( imtype+i, "allfiles" ) ||
+			!strcmp( imtype+i, "allfiles/*" ) ||
+			!strcmp( imtype+i, "allfiles/all" ) ||
+			!strcmp( imtype+i, "all/allfiles" )){
+
+				g_free( im->data );
+				im->data = g_strdup( "all/all" );
+				tmp = g_strdup_printf( "%sfile", positive ? "" : "!" );
+				schemes = g_slist_prepend( schemes, tmp );
+				modified = TRUE;
+		}
+	}
+
+	if( modified ){
+		na_object_set_mimetypes( context, mimetypes );
+		na_object_set_schemes( context, schemes );
+	}
+
+	na_core_utils_slist_free( mimetypes );
+	na_core_utils_slist_free( schemes );
+}
+
 static gboolean
 v_is_candidate( NAIContext *context, guint target, GList *selection )
 {
