@@ -38,13 +38,13 @@
 
 /* private class data
  */
-struct NAObjectClassPrivate {
+struct _NAObjectClassPrivate {
 	void *empty;						/* so that gcc -pedantic is happy */
 };
 
 /* private instance data
  */
-struct NAObjectPrivate {
+struct _NAObjectPrivate {
 	gboolean   dispose_has_run;
 };
 
@@ -167,13 +167,13 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "na_object_instance_init";
 	NAObject *self;
 
-	g_debug( "%s: instance=%p (%s), klass=%p",
-			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
-
 	g_return_if_fail( NA_IS_OBJECT( instance ));
 
 	self = NA_OBJECT( instance );
 
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
+
 	self->private = g_new0( NAObjectPrivate, 1 );
 }
 
@@ -183,14 +183,14 @@ instance_dispose( GObject *object )
 	static const gchar *thisfn = "na_object_instance_dispose";
 	NAObject *self;
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_return_if_fail( NA_IS_OBJECT( object ));
 
 	self = NA_OBJECT( object );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 		self->private->dispose_has_run = TRUE;
 
 		na_iduplicable_dispose( NA_IDUPLICABLE( object ));
@@ -210,10 +210,10 @@ instance_finalize( GObject *object )
 
 	g_return_if_fail( NA_IS_OBJECT( object ));
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	self = NA_OBJECT( object );
 
+	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 	g_free( self->private );
 
 	if( NA_IS_IFACTORY_OBJECT( object )){
@@ -359,7 +359,7 @@ iduplicable_is_valid_iter( GObjectClass *class, const NAObject *a, HierarchyIter
 
 /**
  * na_object_object_check_status:
- * @object: the #NAObject-derived object to be checked.
+ * @object: the #NAObject -derived object to be checked.
  *
  * Recursively checks for the edition status of @object and its childs
  * (if any).
@@ -370,12 +370,14 @@ iduplicable_is_valid_iter( GObjectClass *class, const NAObject *a, HierarchyIter
  * na_object_object_check_status( object )
  *  +- na_iduplicable_check_status( object )
  *      +- get_origin( object )
- *      +- modified_status = v_are_equal( origin, object ) -> interface are_equal()
- *      +- valid_status = v_is_valid( object )             -> interface is_valid()
+ *      +- modified_status = v_are_equal( origin, object ) -> interface <structfield>NAObjectClass::are_equal</structfield>
+ *      +- valid_status = v_is_valid( object )             -> interface <structfield>NAObjectClass::is_valid</structfield>
  *
  * Note that the recursivity is managed here, so that we can be sure
  * that edition status of childs is actually checked before those of
  * the parent.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_check_status( const NAObject *object )
@@ -383,14 +385,14 @@ na_object_object_check_status( const NAObject *object )
 	static const gchar *thisfn = "na_object_object_check_status";
 	GList *children, *ic;
 
-	g_debug( "%s: object=%p (%s)",
-			thisfn,
-			( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_return_if_fail( NA_IS_OBJECT( object ));
 
 	if( !object->private->dispose_has_run ){
 
+		g_debug( "%s: object=%p (%s)",
+				thisfn,
+				( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 		if( NA_IS_OBJECT_ITEM( object )){
 			children = na_object_get_items( object );
 
@@ -410,10 +412,12 @@ na_object_object_check_status( const NAObject *object )
  * Checks for modification and validity status of the @object, its
  * parent, the parent of its parent, etc. up to the top of the hierarchy.
  *
- * Returns: %TRUE if at least one of the status has changed, %FALSE else.
- *
  * Checking the modification of any of the status should be more
  * efficient that systematically force the display of the item.
+ *
+ * Returns: %TRUE if at least one of the status has changed, %FALSE else.
+ *
+ * Since: 2.30
  */
 gboolean
 na_object_object_check_status_up( const NAObject *object )
@@ -474,11 +478,13 @@ push_modified_status_up( const NAObject *object, gboolean is_modified )
 
 /**
  * na_object_object_copy:
- * @target: the target #NAObject-derived object.
- * @source: the source #NAObject-derived object.
+ * @target: the target #NAObject -derived object.
+ * @source: the source #NAObject -derived object.
  * @recursive: whether the copy should be recursive.
  *
  * Copies @source to @target.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_copy( NAObject *target, const NAObject *source, gboolean recursive )
@@ -516,16 +522,18 @@ object_copy_iter( GObjectClass *class, const NAObject *source, CopyIter *data )
 
 /**
  * na_object_object_dump:
- * @object: the #NAObject-derived object to be dumped.
+ * @object: the #NAObject -derived object to be dumped.
  *
- * Dumps via g_debug the actual content of the object.
+ * Dumps via g_debug() the actual content of the object.
  *
- * The recursivity is dealt with here. If we let #NAObjectItem do this,
- * the dump of #NAObjectItem-derived object will be splitted, childs
- * being inserted inside.
+ * The recursivity is dealt with here because, if we would let
+ * #NAObjectItem do this, the dump of #NAObjectItem -derived object
+ * would be splitted, childs being inserted inside.
  *
  * na_object_dump() doesn't modify the reference count of the dumped
  * object.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_dump( const NAObject *object )
@@ -551,11 +559,13 @@ na_object_object_dump( const NAObject *object )
 
 /**
  * na_object_object_dump_norec:
- * @object: the #NAObject-derived object to be dumped.
+ * @object: the #NAObject -derived object to be dumped.
  *
  * Dumps via g_debug the actual content of the object.
  *
  * This function is not recursive.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_dump_norec( const NAObject *object )
@@ -582,9 +592,11 @@ dump_class_hierarchy_iter( GObjectClass *class, const NAObject *object, void *us
 
 /**
  * na_object_object_dump_tree:
- * @tree: a hierarchical list of #NAObject-derived objects.
+ * @tree: a hierarchical list of #NAObject -derived objects.
  *
  * Outputs a brief, hierarchical dump of the provided list.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_dump_tree( GList *tree )
@@ -625,7 +637,7 @@ dump_tree( GList *tree, gint level )
 
 /**
  * na_object_object_reset_origin:
- * @object: a #NAObject-derived object.
+ * @object: a #NAObject -derived object.
  * @origin: must be a duplication of @object.
  *
  * Recursively reset origin of @object and its children to @origin (and
@@ -635,6 +647,8 @@ dump_tree( GList *tree, gint level )
  *
  * This only works if @origin has just been duplicated from @object,
  * and thus we do not have to check if children lists are equal.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_reset_origin( NAObject *object, const NAObject *origin )
@@ -661,12 +675,14 @@ na_object_object_reset_origin( NAObject *object, const NAObject *origin )
 
 /**
  * na_object_object_ref:
- * @object: a #NAObject-derived object.
+ * @object: a #NAObject -derived object.
  *
  * Recursively ref the @object and all its children, incrementing their
  * reference_count by 1.
  *
  * Returns: a reference on the @pbject.
+ *
+ * Since: 2.30
  */
 NAObject *
 na_object_object_ref( NAObject *object )
@@ -674,22 +690,22 @@ na_object_object_ref( NAObject *object )
 	NAObject *ref = NULL;
 	GList *children, *ic;
 
-	g_debug( "na_object_object_ref: object=%p (%s, ref_count=%d)",
-			( void * ) object, G_OBJECT_TYPE_NAME( object ), G_OBJECT( object )->ref_count );
-
 	g_return_val_if_fail( NA_IS_OBJECT( object ), NULL );
 
 	if( !object->private->dispose_has_run ){
 
+		g_debug( "na_object_object_ref: object=%p (%s, ref_count=%d)",
+				( void * ) object, G_OBJECT_TYPE_NAME( object ), G_OBJECT( object )->ref_count );
+
 		if( NA_IS_OBJECT_ITEM( object )){
 
 			children = na_object_get_items( object );
 
 			for( ic = children ; ic ; ic = ic->next ){
-
+				/*
 				g_debug( "na_object_object_ref: child=%p (%s, ref_count=%d)",
 							( void * ) ic->data, G_OBJECT_TYPE_NAME( ic->data ), G_OBJECT( ic->data )->ref_count );
-
+							*/
 				na_object_ref( ic->data );
 			}
 		}
@@ -702,31 +718,38 @@ na_object_object_ref( NAObject *object )
 
 /**
  * na_object_object_unref:
- * @object: a #NAObject-derived object.
+ * @object: a #NAObject -derived object.
  *
  * Recursively unref the @object and all its children, decrementing their
  * reference_count by 1.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_unref( NAObject *object )
 {
-	GList *children, *ic;
-
-	g_debug( "na_object_object_unref: object=%p (%s, ref_count=%d)",
-			( void * ) object, G_OBJECT_TYPE_NAME( object ), G_OBJECT( object )->ref_count );
+	GList *children;
+	GList *ic, *icnext;
 
 	g_return_if_fail( NA_IS_OBJECT( object ));
 
 	if( !object->private->dispose_has_run ){
 
+		g_debug( "na_object_object_unref: object=%p (%s, ref_count=%d)",
+				( void * ) object, G_OBJECT_TYPE_NAME( object ), G_OBJECT( object )->ref_count );
+
 		if( NA_IS_OBJECT_ITEM( object )){
 			children = na_object_get_items( object );
 
-			for( ic = children ; ic ; ic = ic->next ){
-				na_object_unref( ic->data );
+			for( ic = children ; ic ; ic = icnext ){
+				icnext = ic->next;
+				g_object_unref( ic->data );
 			}
 		}
 
+		/*g_debug( "na_object_object_unref:about_to_unref: object=%p (%s, ref_count=%d)",
+				( void * ) object, G_OBJECT_TYPE_NAME( object ), G_OBJECT( object )->ref_count );*/
+
 		g_object_unref( object );
 	}
 }
@@ -781,9 +804,12 @@ build_class_hierarchy( const NAObject *object )
 
 /**
  * na_object_object_get_hierarchy:
+ * @object: the #NAObject -derived object.
  *
- * Returns the class hierarchy,
+ * Returns: the class hierarchy,
  * from the topmost base class, to the most-derived one.
+ *
+ * Since: 2.30
  */
 GList *
 na_object_object_get_hierarchy( const NAObject *object )
@@ -804,6 +830,12 @@ na_object_object_get_hierarchy( const NAObject *object )
 
 /**
  * na_object_free_hierarchy:
+ * @hierarchy: the #GList of hierarchy, as returned from
+ *  na_object_get_hierarchy().
+ *
+ * Releases the #NAObject hierarchy.
+ *
+ * Since: 2.30
  */
 void
 na_object_free_hierarchy( GList *hierarchy )
@@ -813,10 +845,12 @@ na_object_free_hierarchy( GList *hierarchy )
 
 /**
  * na_object_object_debug_invalid:
- * @object: the #NAObject-derived object which is invalid.
+ * @object: the #NAObject -derived object which is invalid.
  * @reason: the reason.
  *
  * Dump the object with the invalidity reason.
+ *
+ * Since: 2.30
  */
 void
 na_object_object_debug_invalid( const NAObject *object, const gchar *reason )
