@@ -85,6 +85,7 @@ struct NAXMLReaderPrivate {
 	 */
 	gboolean                       type_found;
 	GList                         *nodes;
+	GList                         *dealt;
 	RootNodeStr                   *root_node_str;
 	gchar                         *item_id;
 
@@ -158,12 +159,10 @@ static RootNodeStr st_root_node_str[] = {
 static void          read_start_profile_attach_profile( NAXMLReader *reader, NAObjectProfile *profile );
 static gboolean      read_data_is_path_adhoc_for_object( NAXMLReader *reader, const NAIFactoryObject *object, xmlChar *text );
 static NADataBoxed  *read_data_boxed_from_node( NAXMLReader *reader, xmlChar *text, xmlNode *parent, const NADataDef *def );
-static void          read_done_item( NAXMLReader *reader, NAObjectItem *item );
 static void          read_done_item_set_localized_icon( NAXMLReader *reader, NAObjectItem *item );
-static void          read_done_action( NAXMLReader *reader, NAObjectAction *action );
+static void          read_done_action_read_profiles( NAXMLReader *reader, NAObjectAction *action );
 static gchar        *read_done_action_get_next_profile_id( NAXMLReader *reader );
 static void          read_done_action_load_profile( NAXMLReader *reader, const gchar *profile_id );
-static void          read_done_profile( NAXMLReader *reader, NAObjectProfile *profile );
 static void          read_done_profile_set_localized_label( NAXMLReader *reader, NAObjectProfile *profile );
 
 static guint         reader_parse_xmldoc( NAXMLReader *reader );
@@ -252,6 +251,7 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	self->private->parms = NULL;
 	self->private->type_found = FALSE;
 	self->private->nodes = NULL;
+	self->private->dealt = NULL;
 	self->private->root_node_str = NULL;
 }
 
@@ -270,6 +270,7 @@ instance_dispose( GObject *object )
 		self->private->dispose_has_run = TRUE;
 
 		g_list_free( self->private->nodes );
+		g_list_free( self->private->dealt );
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
@@ -606,6 +607,10 @@ read_start_profile_attach_profile( NAXMLReader *reader, NAObjectProfile *profile
 /*
  * this callback function is called by NAIFactoryObject once for each
  * serializable data for the object
+ *
+ * Note that some nodes may be readen twice because of multiple definition
+ * of the same data (e.g. icon which exists in localized and unlocalized
+ * versions). So do not remove dealt-with nodes here
  */
 NADataBoxed *
 naxml_reader_read_data( const NAIFactoryProvider *provider, void *reader_data, const NAIFactoryObject *object, const NADataDef *def, GSList **messages )
@@ -651,7 +656,7 @@ naxml_reader_read_data( const NAIFactoryProvider *provider, void *reader_data, c
 	}
 
 	if( boxed ){
-		reader->private->nodes = g_list_remove( reader->private->nodes, parent_node );
+		reader->private->dealt = g_list_prepend( reader->private->dealt, parent_node );
 	}
 
 	return( boxed );
@@ -750,26 +755,20 @@ naxml_reader_read_done( const NAIFactoryProvider *provider, void *reader_data, c
 			( void * ) messages );
 
 	if( NA_IS_OBJECT_ITEM( object )){
-		read_done_item( NAXML_READER( reader_data ), NA_OBJECT_ITEM( object ));
+		read_done_item_set_localized_icon( NAXML_READER( reader_data ), NA_OBJECT_ITEM( object ));
 	}
 
 	if( NA_IS_OBJECT_ACTION( object )){
-		read_done_action( NAXML_READER( reader_data ), NA_OBJECT_ACTION( object ));
+		read_done_action_read_profiles( NAXML_READER( reader_data ), NA_OBJECT_ACTION( object ));
 	}
 
 	if( NA_IS_OBJECT_PROFILE( object )){
-		read_done_profile( NAXML_READER( reader_data ), NA_OBJECT_PROFILE( object ));
+		read_done_profile_set_localized_label( NAXML_READER( reader_data ), NA_OBJECT_PROFILE( object ));
 	}
 
 	g_debug( "%s: quitting for %s at %p", thisfn, G_OBJECT_TYPE_NAME( object ), ( void * ) object );
 }
 
-static void
-read_done_item( NAXMLReader *reader, NAObjectItem *item )
-{
-	read_done_item_set_localized_icon( reader, item );
-}
-
 /*
  * just having readen this NAObjectItem
  * so deals with unlocalized/localized icon name/path
@@ -805,7 +804,7 @@ read_done_item_set_localized_icon( NAXMLReader *reader, NAObjectItem *item )
  * Also note that profiles order has been introduced in 2.29 serie
  */
 static void
-read_done_action( NAXMLReader *reader, NAObjectAction *action )
+read_done_action_read_profiles( NAXMLReader *reader, NAObjectAction *action )
 {
 	GSList *order, *ip;
 	gchar *profile_id;
@@ -890,12 +889,6 @@ read_done_action_load_profile( NAXMLReader *reader, const gchar *profile_id )
 			&reader->private->parms->messages );
 }
 
-static void
-read_done_profile( NAXMLReader *reader, NAObjectProfile *profile )
-{
-	read_done_profile_set_localized_label( reader, profile );
-}
-
 /*
  * just having readen this NAObjectProfile
  * so deals with unlocalized/localized desc-name
@@ -1075,6 +1068,7 @@ schema_read_value( NAXMLReader *reader, xmlNode *node, const NADataDef *def )
 		value = get_value_from_child_node( node, NAXML_KEY_SCHEMA_NODE_DEFAULT );
 	}
 
+	/*g_debug( "name=%s, localizable=%s, value=%s", def->name, def->localizable ? "True":"False", value );*/
 	return( value );
 }
 
@@ -1380,9 +1374,12 @@ publish_undealt_nodes( NAXMLReader *reader )
 
 	for( iter = reader->private->nodes ; iter ; iter = iter->next ){
 		xmlNode *node = ( xmlNode * ) iter->data;
-		text = xmlNodeGetContent( node );
-		na_core_utils_slist_add_message( &reader->private->parms->messages, WARN_UNDEALT_NODE, ( const gchar * ) text, node->line );
-		xmlFree( text );
+
+		if( !g_list_find( reader->private->dealt, node )){
+			text = xmlNodeGetContent( node );
+			na_core_utils_slist_add_message( &reader->private->parms->messages, WARN_UNDEALT_NODE, ( const gchar * ) text, node->line );
+			xmlFree( text );
+		}
 	}
 }
 
