@@ -109,9 +109,6 @@ nautilus_module_shutdown( void )
 /*
  * a log handler that we install when in development mode in order to be
  * able to log plugin runtime
- * TODO: the debug flag should be dynamic, so that an advanced user could
- * setup a given key and obtain a full log to send to Bugzilla..
- * For now, is always install when compiled in maintainer mode, never else
  */
 static void
 set_log_handler( void )
@@ -141,7 +138,9 @@ log_handler( const gchar *log_domain, GLogLevelFlags log_level, const gchar *mes
 	/*( *st_default_log_func )( log_domain, log_level, message, user_data );*/
 	syslog( LOG_USER | LOG_DEBUG, "%s%s", tmp, message );
 #else
-	/* do nothing */
+	if( g_getenv( NAUTILUS_ACTIONS_DEBUG )){
+		syslog( LOG_USER | LOG_DEBUG, "%s%s", tmp, message );
+	}
 #endif
 
 	g_free( tmp );
